@@ -44,7 +44,7 @@
 #include <exchange-mapi-connection.h>
 #include <exchange-mapi-utils.h>
 
-#define d(x) x
+#define d(x)
 
 gint e_plugin_lib_enable (EPlugin *ep, gint enable);
 
@@ -75,7 +75,7 @@ free_mapi_listener ( void )
 gint
 e_plugin_lib_enable (EPlugin *ep, gint enable)
 {
-	g_debug ("Loading Exchange MAPI Plugin \n");
+	d(g_debug ("Loading Exchange MAPI Plugin"));
 
 	if (!config_listener) {
 		config_listener = exchange_mapi_account_listener_new ();
@@ -377,7 +377,7 @@ org_gnome_exchange_mapi_account_setup (EPlugin *epl, EConfigHookItemFactoryData
 
 		auth_button = gtk_button_new_with_mnemonic (_("_Authenticate"));
 		gtk_box_pack_start (GTK_BOX (hbox), auth_button, FALSE, FALSE, 0);
-		g_signal_connect(GTK_OBJECT(auth_button), "clicked",  G_CALLBACK(validate_credentials), data->config);
+		g_signal_connect (auth_button, "clicked",  G_CALLBACK (validate_credentials), data->config);
 
 		gtk_table_attach (GTK_TABLE (data->parent), label, 0, 1, row, row+1, 0, 0, 0, 0);
 		gtk_widget_show_all (GTK_WIDGET (hbox));
@@ -797,6 +797,8 @@ exchange_mapi_book_commit (EPlugin *epl, EConfigTarget *target)
 
 	e_source_set_property (source, "completion", "true");
 	e_source_set_property (source, "public", "no");
+	e_source_set_property (source, "folder-id", sfid);
+
 	// Update the folder list in the plugin and ExchangeMAPIFolder
 	g_free (r_uri);
 	g_free (sfid);
